diff --git a/share/translations/keepassxc_en.ts b/share/translations/keepassxc_en.ts
index 4df6cef14..b8f34456c 100644
--- a/share/translations/keepassxc_en.ts
+++ b/share/translations/keepassxc_en.ts
@@ -644,6 +644,10 @@
+
+
+
+
AutoTypeAssociationsModel
diff --git a/tests/TestAutoType.cpp b/tests/TestAutoType.cpp
index 235ba5a86..445735f9f 100644
--- a/tests/TestAutoType.cpp
+++ b/tests/TestAutoType.cpp
@@ -337,7 +337,7 @@ void TestAutoType::testAutoTypeSyntaxChecks()
QVERIFY2(AutoType::verifyAutoTypeSyntax("{S:FOO}{S:HELLO WORLD}", entry, error), error.toLatin1());
QVERIFY2(!AutoType::verifyAutoTypeSyntax("{S:SPECIAL_TOKEN{}}", entry, error), error.toLatin1());
- QVERIFY2(!AutoType::verifyAutoTypeSyntax("{BEEP 3 3}", entry, error), error.toLatin1());
+ QVERIFY2(AutoType::verifyAutoTypeSyntax("{BEEP 3 3}", entry, error), error.toLatin1());
QVERIFY2(AutoType::verifyAutoTypeSyntax("{BEEP 3}", entry, error), error.toLatin1());
QVERIFY2(AutoType::verifyAutoTypeSyntax("{VKEY 0x01}", entry, error), error.toLatin1());
diff --git a/src/autotype/AutoType.cpp b/src/autotype/AutoType.cpp
index 57299fb66..750053955 100644
--- a/src/autotype/AutoType.cpp
+++ b/src/autotype/AutoType.cpp
@@ -647,13 +647,26 @@ AutoType::parseSequence(const QString& entrySequence, const Entry* entry, QStrin
for (const auto& ch : totp) {
actions << QSharedPointer::create(ch);
}
- } else if (placeholder == "pickchars") {
- // Ignore this if we are syntax checking
+ } else if (placeholder.startsWith("pickchars")) {
+ // Reset to the original capture to preserve case
+ placeholder = match.captured(3);
+
+ auto attribute = EntryAttributes::PasswordKey;
+ if (placeholder.contains(":")) {
+ attribute = placeholder.section(":", 1);
+ if (!entry->attributes()->hasKey(attribute)) {
+ error = tr("Entry does not have attribute for PICKCHARS: %1").arg(attribute);
+ return {};
+ }
+ }
+
+ // Bail out if we are just syntax checking
if (syntaxOnly) {
continue;
}
- // Show pickchars dialog for entry's password
- auto password = entry->resolvePlaceholder(entry->password());
+
+ // Show pickchars dialog for the desired attribute
+ auto password = entry->resolvePlaceholder(entry->attribute(attribute));
if (!password.isEmpty()) {
PickcharsDialog pickcharsDialog(password);
if (pickcharsDialog.exec() == QDialog::Accepted && !pickcharsDialog.selectedChars().isEmpty()) {
@@ -746,8 +759,8 @@ AutoType::parseSequence(const QString& entrySequence, const Entry* entry, QStrin
mode = AutoTypeExecutor::Mode::VIRTUAL;
}
actions << QSharedPointer::create(mode);
- } else if (placeholder == "beep" || placeholder.startsWith("vkey") || placeholder.startsWith("appactivate")
- || placeholder.startsWith("c:")) {
+ } else if (placeholder.startsWith("beep") || placeholder.startsWith("vkey")
+ || placeholder.startsWith("appactivate") || placeholder.startsWith("c:")) {
// Ignore these commands
} else {
// Attempt to resolve an entry attribute